// neopixel.ino --- 16.09.2018 -----------------------------------------

// Status-Anzeige mit Neopixelstring, über MQTT angesteuert
//
// 1.) Payload hexadezimal:
// genau (!) 6 Byte, im 6. Byte muss Bit.7 gesetzt sein:
// Start-LED, Rot, Grün, Blau, End-LED, Blinkmodus
//
// 2.) Payload als Komma-separierte-Liste:
// (nur Ziffern und Kommata werden gelesen !)
// mindestens 1 Byte erforderlich, alle fehlenden Bytes werden
// auf '0' gesetzt
// Start-LED[, Rot[, Grün[, Blau[, End-LED[, Blinkmode]]]]]
// Beispiel:
// "0" -> "0,0,0,0,0,0" -> löscht LED '0'
// "5,255" ->  "5,255,0,0,0,0" -> setzt LED '5' auf Rot
// "10,,,255,15" -> "10,0,0,255,15,0" -> LED 10-14 auf Blau
// "15,,255,,1" -> "15,0,255,0,0,1" -> LED 15 blinkt mit 1Hz
// "0,,,,16" -> "0,0,0,0,16,0" -> Löscht LED 0 bis 15
//
// Blinkmodus:
// 1 -> 1Hz
// 2 -> 2Hz
// 3 -> 4Hz
// 4 -> 1Hz (asymmetrisch: 125ms an, 875ms aus)


#ifdef NEOPIXEL_STATIC

// Max. Anzahl der durch Kommo separierten Parameter in der Payload
// Start[, R[, G[, B[, End[, Blink]]]]]
#define MAX_BYTES       6
uint8_t rx[MAX_BYTES];       // hier werden die empfangenen Bytes abgelegt
#define CPAYLOAD_LENGTH 30   // nur zum Debuggen benötigt

#ifdef NEOPIXEL_BLINK
uint8_t blink_mode[NEOPIXELS];
uint8_t blink_r[NEOPIXELS];
uint8_t blink_g[NEOPIXELS];
uint8_t blink_b[NEOPIXELS];
#define BLINKMODES 4         // Anzahl Blinkmuster
#endif

// --------------------------------------------------------------
void neopixel(byte* payload, uint length) 
{
  uint8_t i, j;

  if (length == 0) return;     // no length, no fun

#ifdef DEBUG
  // payload nach cpayload kopieren und anzeigen
  char  cpayload[CPAYLOAD_LENGTH];
  for (i = 0; i < length; i++) cpayload [i] = (char) payload[i];
  cpayload[i] = 0;

  Serial.print("payload: ");
  Serial.print(cpayload);
  Serial.print(", Länge : ");
  Serial.print(length);
  if (length > 5)
  {
  	Serial.print(", Payload[5] : ");
  	Serial.print(payload[5]);
  }
  Serial.println();
#endif

  // alle rx-Bytes auf '0' setzen
  for (i = 0; i < MAX_BYTES; i++) rx[i] = 0;

  // ------------- Payload wird binär übertragen
  if ((length == 6) && (payload[5] & 0x80))
  {
    for(i = 0; i < length; i++)
    {
      rx[i] = payload[i];
    }
   // das Bit.7 als Indikator für Binäre Übertragung entfernen
    rx[5] = rx[5] & 0x0F;
  }

  // --------------- Payload via ASCII-Daten
  else
  {
    j = 0;
    for (i = 0; i < length; i++)
    {
      if (isDigit(( (char) payload[i]) ))
      {
        rx[j] *= 10;
        rx[j] += (payload[i] - '0');
      }
      // nächstes Byte
      else if (payload[i] == ',') j++;

    #ifdef DEBUG
      else       // not isDigit()
      {
        Serial.print((char) payload[i]);
        Serial.println();
      }
    #endif

      if (j >= MAX_BYTES) break;
    }
  }
#ifdef DEBUG
  Serial.print(" ");
  for (i = 0; i < MAX_BYTES; i++) 
  {
    Serial.print(rx[i]);
    Serial.print("  ");
  }
  Serial.println();
#endif

  set_mypixel(rx);
}


// ------------------------------------------------------------
// hier kann der mQTT-Client direkt seine LED's setzen
// ohne den Umweg über den Server gehen zu müssen
void set_mypixel(uint8_t * rx)
{
  uint8_t i, start, end;
  start = rx[0];
  if (start > NEOPIXELS - 1) start = NEOPIXELS - 1;
    
  end = rx[4];
  if (end <= start) end = start + 1;  // genau 1 Pixel
  else end++;    // end hinter das letzte Pixel setzen

  if (end > NEOPIXELS) end = NEOPIXELS ;

#ifdef DEBUG
  Serial.print("Start: ");
  Serial.print(start);
  Serial.print(", Ende: ");
  Serial.print(end);
  Serial.print(", r: ");
  Serial.print(rx[1]);
  Serial.print(", g: ");
  Serial.print(rx[2]);
  Serial.print(", b: ");
  Serial.print(rx[4]);
  Serial.print(", f: ");
  Serial.print(rx[5]);
  Serial.println();
#endif

  for (i = start; i < end; i++)
  {
  #ifdef NEOPIXEL_BLINK
    if (rx[5] == 0)      // kein Blinken 
    {
      pixels.setPixelColor(i, pixels.Color(rx[1], rx[2], rx[3]));
      blink_mode[i] = 0;
    }
    else                 // Blinken
    {
      if (rx[5] > BLINKMODES) rx[5] = BLINKMODES;
      blink_mode[i] = rx[5];
      blink_r[i] = rx[1]; blink_g[i] = rx[2]; blink_b[i] = rx[3];
    }
 #else  
    pixels.setPixelColor(i, pixels.Color(rx[1], rx[2], rx[3]));
 #endif 

  } 
#ifdef NEOPIXEL_BLINK
  // wenn gerade eine LED auf Blinken geschaltet wurde, 
  // dann wird sie durch check_blink() zeitsynchron geschaltet.
  if (rx[5] == 0) pixels.show();
#else
  pixels.show();
#endif
}


/*
// --------------------------------------------------------------
// auf dem aktuellen Client direkt ein einzelnes NeoPixel setzen
// pixel[0] = LED-Nr.
// pixel[1] = rot, pixel[2] = grün, pixel[3] = blau, 
// pixel[4] = Blinkmodus
void set_mypixel(uint8_t* pixel)
{
#ifdef NEOPIXEL_BLINK
  if (pixel[4])            // Blinkmodus, wenn > 0
  {
  	blink_r[pixel[0]]    = pixel[1];
  	blink_g[pixel[0]]    = pixel[2];
  	blink_b[pixel[0]]    = pixel[3];
  	blink_mode[pixel[0]] = pixel[4];
  }
  else                     // Dauerlicht, wenn 0
  {
  	pixels.setPixelColor(pixel[0], pixels.Color(pixel[1], pixel[2], pixel[3]));
  	pixels.show();
  }
#else
  pixels.setPixelColor(pixel[0], pixels.Color(pixel[1], pixel[2], pixel[3]));
  pixels.show();
#endif
}

*/


// --------------------------------------------------------
#ifdef NEOPIXEL_BLINK
// wird alle 125ms aufgerufen und prüft jede einzelne LED darauf,
// ob der Blinkmodus aktiviert ist (blink_mode[i] > 0).
// LED       : -> an, . -> aus
// phase     0  1  2  3  4  5  6  7 
// blink = 3 :  .  :  .  :  .  :  .  -> 4 Hz
//       = 2 :     .     :     .     -> 2 Hz
//       = 1 :           .           -> 1 Hz
//       = 4 :  .                    -> 1 Hz (125ms +  875ms)

void check_blink(void)
{
  static uint8_t phase = 0;
  uint8_t refresh_string = false;
  uint8_t refresh_pixel = false;
  uint8_t r, g, b, i;

  for (i = 0; i < NEOPIXELS; i++)
  {

    // wenn das Flag für Blinken gesetzt ist
    if (blink_mode[i])
    {    
      refresh_pixel = false; 
      // blink == 3  / 4 Hz
      if ( (blink_mode[i] == 3) && (phase & 0x01) )
      {
          r = 0; g = 0; b = 0;
          refresh_pixel = true;
      }
      else if ( (blink_mode[i] == 3) && (phase & 0x01) == 0)
      {
          r = blink_r[i]; g = blink_g[i]; b = blink_b[i];
          refresh_pixel = true;
      }

      // ------------ blink == 2 / 2 Hz
      else if ((blink_mode[i] == 2) && (phase & 0x02) == 0)
      {
          r = 0; g = 0; b = 0;
          refresh_pixel = true;
      }
      else if ((blink_mode[i] == 2) && (phase & 0x02) == 2)
      {
          r = blink_r[i]; g = blink_g[i]; b = blink_b[i];
          refresh_pixel = true;
      }

      // ------------  blink == 1 / 1 Hz
      else if ((blink_mode[i] == 1) && (phase == 4))
      {
          r = 0; g = 0; b = 0;
          refresh_pixel = true;
      }
      else if ((blink_mode[i] == 1) && (phase == 0) )
      {
          r = blink_r[i]; g = blink_g[i]; b = blink_b[i];
          refresh_pixel = true;
      }

      // ------------  blink == 4 / 1 Hz  125 / 875ms
      else if ((blink_mode[i] == 4) && (phase == 1))
      {
          r = 0; g = 0; b = 0;
          refresh_pixel = true;
      }
      else if ((blink_mode[i] == 4) && (phase == 0) )
      {
          r = blink_r[i]; g = blink_g[i]; b = blink_b[i];
          refresh_pixel = true;
      }

      if (refresh_pixel)
      {
          refresh_string = true;
          pixels.setPixelColor(i, pixels.Color(r, g, b));
      }
    }  
    
    if (refresh_string) pixels.show();
  }

  phase++;
  if (phase == 8) phase = 0;
}

#endif  // NEOPIXEL_BLINK
#endif  // NEOPOXEL_STATIC
